
#include <iostream>
#include <cmath>

#include "vec.hh"
#include "ray.hh"
#include "mirror.hh"

mirror::mirror( void ) {
}

mirror::mirror( const vec& _normal, double _offset ) {
    this->set( _normal, _offset );
}

mirror::mirror( const mirror& _orig ) {
    this->set( _orig.normal, _orig.offset );
}

mirror& mirror::operator = ( const mirror& _orig ) {
    this->set( _orig.normal, _orig.offset );
    return *this;
}

bool mirror::checkRayIntersection( const ray& _ray, double* _distance ) const {
    double dn = this->normal.dot( _ray.getDirection() );

    if ( std::fabs( dn ) > 0.0001 ) {
	double xn = this->normal.dot( _ray.getStart() );

	double dd = ( this->offset - xn ) / dn;

	if ( dd >= 0.0001 ) {
	    if ( _distance != 0 ) {
		*_distance = dd;
	    }
	    return true;
	}
    }

    return false;
}

bool mirror::reflect( const ray& _in, ray* _out, double* _distance ) const {
    double dd;

    if ( this->checkRayIntersection( _in, &dd ) ) {
	if ( _distance != 0 ) {
	    *_distance = dd;
	}

	if ( _out != 0 ) {
	    vec oldDir( _in.getDirection() );

	    vec start( oldDir );
	    start *= dd;
	    start += _in.getStart();

	    vec dir( this->normal );
	    dir *= -2.0 * this->normal.dot( oldDir );
	    dir += oldDir;

	    *_out = ray( start, dir );
	}

	return true;
    }

    return false;
}

void mirror::set( const vec& _normal, double _offset ) {
    this->normal = _normal;
    this->offset = _offset;
}
